/* dc_gui2 - a GTK+2 GUI for DCTC
 * Copyright (C) 2002 Eric Prevoteau
 *
 * flagged_user_clist.c: Copyright (C) Eric Prevoteau <www@ac2i.tzo.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
$Id: flagged_user_clist.c,v 1.2 2003/10/26 08:48:09 ericprev Exp $
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <db.h>
#include <gnome.h>

#include "gtkcellrenderertextx.h"
#include "misc_gtk.h"
#include "flagged_user_clist.h"
#include "main.h"
#include "gui_define.h"
#include "macro.h"
#include "unode.h"
#include "init_fnc.h"
#include "bdb.h"

/*****************************************/
/* reload the flagged user clist content */
/*****************************************/
void reload_flagged_user_clist(int only_if_empty)
{
	GtkWidget *w;
	w=get_widget_by_widget_name(main_window,"flagged_user_clist");
	if(w)
	{
		GtkListStore *gls;
		GtkTreeIter iter;
		GtkTreeModel *gtm;
		DBC *cursor;
		int ret;

		gls=GTK_LIST_STORE(gtm=gtk_tree_view_get_model(GTK_TREE_VIEW(w)));

		if((only_if_empty==1)&&gtk_tree_model_is_not_empty(gtm))
			return;

		gtk_list_store_clear(gls);

		ret=unwanted_user->cursor(unwanted_user,NULL,&cursor,0);
		if(ret==0)
		{
			DBT key;
			DBT data;
			char buf1[8192];
			char buf2[8192];

			memset(&key,0,sizeof(key));
			memset(&data,0,sizeof(data));

			key.data=buf1;
			key.ulen=sizeof(buf1)-1;
			key.flags=DB_DBT_USERMEM;

			data.data=buf2;
			data.ulen=sizeof(buf2)-1;
			data.flags=DB_DBT_USERMEM;

			ret=cursor->c_get(cursor,&key,&data,DB_FIRST);
			while(ret==0)
			{
				buf1[key.size]='\0';
				buf2[data.size]='\0';

				gtk_list_store_append(gls,&iter);
				if(utf8_mode==TRUE)
				{
					gtk_list_store_set(gls,&iter,FUC_NICK_COL,buf1,
															FUC_FLAGS_COL,buf2,
															FUC_NICK,buf1,
															-1);
				}
				else
				{
					gchar *utf8;
					utf8=g_locale_to_utf8(buf1,-1,NULL,NULL,NULL);
					gtk_list_store_set(gls,&iter,FUC_NICK_COL,utf8,
															FUC_FLAGS_COL,buf2,
															FUC_NICK,buf1,
															-1);
					g_free(utf8);
				}

				ret=cursor->c_get(cursor,&key,&data,DB_NEXT);
			}

			/* end of scan, destroy the cursor */
			cursor->c_close(cursor);
		}
	}
}

/****************************************************/
/* build modele and view for the flagged user clist */
/****************************************************/
void bmav4_flagged_user_clist(void)
{
	GtkListStore *model;
	GtkWidget *view;
	GtkCellRenderer *rend;
	GtkTreeSelection *slc;

	model=gtk_list_store_new(NB_FUC_COL,G_TYPE_STRING,G_TYPE_STRING, G_TYPE_STRING);

	view=get_widget_by_widget_name(main_window,"flagged_user_clist");

	gtk_tree_view_set_model(GTK_TREE_VIEW(view),GTK_TREE_MODEL(model));
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view),TRUE);

	slc=gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
	gtk_tree_selection_set_mode(slc,GTK_SELECTION_SINGLE);
	
	/* the first column of the view display the first column of the model, and so on. The 4th entry of the model is not displayed */
	rend=gtk_cell_renderer_textx_new();
	g_object_set(rend,"rowspacing",TRUE,NULL);
	MY_RS_gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),FUC_NICK_COL,_("Nick"),rend,"text",FUC_NICK_COL,NULL);

	rend=gtk_cell_renderer_textx_new();
	g_object_set(rend,"rowspacing",TRUE,NULL);
	MY_RS_gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),FUC_FLAGS_COL,_("Flags"),rend,"text",FUC_FLAGS_COL,NULL);

	/* The view now holds a reference.  We can get rid of our own
	 * reference */
	g_object_unref (G_OBJECT (model));
}

