/* dc_gui2 - a GTK+2 GUI for DCTC
 * Copyright (C) 2002 Eric Prevoteau
 *
 * misc.c: Copyright (C) Eric Prevoteau <www@ac2i.tzo.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
$Id: misc.c,v 1.6 2003/10/19 08:44:37 ericprev Exp $
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <ctype.h>
#include <glib.h>

#include "misc.h"

/***********************************************************/
/* routine copiant par troncature une chaine de caracteres */
/***********************************************************/
void strncpy_max(char *destination, const char *source, int max_byte)
{
	strncpy(destination,source,max_byte-1);
	destination[max_byte-1]='\0';
}

/************************************************************************/
/* check if the given pattern exists inside filename (case insensitive) */
/************************************************************************/
/* output: address of the pattern or NULL */
/******************************************/
char *my_strcasestr(GString *pattern, char *filename)
{
	int max_pos;
	int i;

	max_pos=strlen(filename)-pattern->len;
	if(max_pos<0)				 /* filename length < pattern length ? */
		return NULL;				/* no match */

	for(i=0;i<=max_pos;i++)
	{
		if(!strncasecmp(pattern->str,filename+i,pattern->len))
			return filename+i;
	}

	return NULL;					/* no match */
}

/***************************************************/
/* compute the size of a NULL terminated ptr array */
/***************************************************/
int size_of_null_array(void **array) 
{
	int nb=0;
 
	while(*array!=NULL)
	{
		nb++;
		array++;
	}
	return nb;
}

/*
----------------------
----------------------
*/

static guint8 ascii1_to_hex(unsigned char v)
{
   if((v>='0')&&(v<='9'))
      return v-'0';
   else
      return toupper(v)-'A'+10;
}

/******************************************/
/* convert 2 ascii hexdigit into 1 guint8 */
/******************************************/
guint8 ascii2_to_bin1(unsigned char *asc)
{
   return ((ascii1_to_hex(asc[0]))<<4)|(ascii1_to_hex(asc[1]));
}

/*************************************************************/
/* build any required directory from the given absolute path */
/*************************************************************/
static int recursive_build_directory(GString *dir)
{
	struct stat st;

	if(stat(dir->str,&st)==0)
	{
		return 0;
	}
	else
	{
		GString *new_dir;
		char *t;
		int ret;

		new_dir=g_string_new(dir->str);
		t=strrchr(new_dir->str,'/');
		if(t==NULL)
		{
			g_string_free(new_dir,TRUE);
			return ENOTDIR;
		}

		new_dir=g_string_truncate(new_dir,t-new_dir->str);
		ret=recursive_build_directory(new_dir);
		g_string_free(new_dir,TRUE);

		if(ret!=0)
			return ret;

		if(stat(dir->str,&st)==0)
		{	/* it can look stupid to check if the directory we want to create already exist */
			/* because we just have created its parent however, if the directory we want to create is "xxx/yyy/." */
			/* creating "xxx/yyy" has already created the "." directory. */
			return 0;
		}

		if(mkdir(dir->str,0777))
		{
			return errno;
		}
		return 0;
	}
}

/******************************************************/
/* create the given directory (it can start with a /) */
/******************************************************/
/* output: 0=ok else errno */
/***************************/
int recursive_mkdir(GString *directory_name)
{
	GString *final_name;
	int ret;

	if(directory_name->str[0]=='/')
		final_name=g_string_new(directory_name->str);
	else
	{
		char the_cwd[MAXPATHLEN];
		if(getcwd(the_cwd,sizeof(the_cwd)-1)==NULL)
		{
			return errno;
		}

		final_name=g_string_new(the_cwd); 
		if(final_name->str[final_name->len-1]!='/')
			final_name=g_string_append_c(final_name,'/'); 
		final_name=g_string_append(final_name,directory_name->str);
	}

	/* now, final_name is an absolute path */
	ret=recursive_build_directory(final_name);

	g_string_free(final_name,TRUE);

	return ret;
}

/*********************************************/
/* check if the given program is in the path */
/*********************************************/
/* output: TRUE=yes, FALSE=no */
/******************************/
gboolean check_prog_in_path(const char *progname)
{
	GString *str;
	char *path;
	gchar **row;
	gboolean output=FALSE;
	int i=0;

	if(progname[0]=='/')
	{
		if(access(progname,X_OK)==0)
			return TRUE;
		return output;
	}
	
	path=getenv("PATH");
	if(path==NULL)
		return output;

	row=g_strsplit(path,":",0);
	str=g_string_new("");

	while(row[i]!=NULL)
	{
		g_string_assign(str,row[i]);
		g_string_append_c(str,'/');
		g_string_append(str,progname);

		if(access(str->str,X_OK)==0)
		{
			output=TRUE;
			break;
		}
		else
			i++;
	}

	g_string_free(str,TRUE);
	g_strfreev(row);
	return output;
}

